/**
 * Copyright French Prime minister Office/SGMAP/DINSIC/Vitam Program (2019-2020)
 * and the signatories of the "VITAM - Accord du Contributeur" agreement.
 *
 * contact@programmevitam.fr
 *
 * This software is a computer program whose purpose is to implement
 * implement a digital archiving front-office system for the secure and
 * efficient high volumetry VITAM solution.
 *
 * This software is governed by the CeCILL-C license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL-C
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability.
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL-C license and that you accept its terms.
 */
package fr.gouv.vitamui.commons.logbook.scheduler;

import java.time.OffsetDateTime;
import java.util.Collection;

import javax.annotation.PostConstruct;

import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.mongodb.core.query.Criteria;
import org.springframework.scheduling.annotation.Scheduled;

import fr.gouv.vitamui.commons.api.logger.VitamUILogger;
import fr.gouv.vitamui.commons.api.logger.VitamUILoggerFactory;
import fr.gouv.vitamui.commons.logbook.common.EventStatus;
import fr.gouv.vitamui.commons.logbook.dao.EventRepository;
import fr.gouv.vitamui.commons.logbook.domain.Event;
import lombok.Getter;
import lombok.Setter;

/**
 *
 * Task who are responsable to delete events who are already in the logbook vitam.
 *
 *
 */
@Getter
@Setter
public class DeleteSynchronizedEventsTasks {

    private final EventRepository eventRepository;

    @Value("${logbook.scheduling.deleteSynchronizedEventsTasks.cronExpression}")
    private String cronExpression;

    /**
     * By default every 30 days, events sent with success to vitam is deleted
     */
    @Value("${logbook.scheduling.deleteSynchronizedEventsTasks.ttlInDays:30}")
    private Long ttlInDays;

    private static final VitamUILogger LOGGER = VitamUILoggerFactory.getInstance(DeleteSynchronizedEventsTasks.class);

    public DeleteSynchronizedEventsTasks(final EventRepository eventRepository) {
        this.eventRepository = eventRepository;
    }

    @PostConstruct
    public void init() {
        LOGGER.info(
                "deleteSynchronizedEventsTasks is running with cron expression {}, deleting events synchronized from {} days ",
                cronExpression, ttlInDays);
    }

    @Scheduled(cron = "${logbook.scheduling.deleteSynchronizedEventsTasks.cronExpression}")
    public void run() {
        LOGGER.debug("deleteSynchronizedEventsTasks is started");
        Collection<Event> events = getEventsElligibleToBeDeleted();
        events.forEach(e -> {
            LOGGER.trace("Delete event {}", e);
            eventRepository.deleteById(e.getId());
        });
        LOGGER.debug("deleteSynchronizedEventsTasks is done");

    }

    protected Collection<Event> getEventsElligibleToBeDeleted() {
        OffsetDateTime now = OffsetDateTime.now();
        now = now.minusDays(ttlInDays);
        return eventRepository
                .findAll(Criteria.where("status").is(EventStatus.SUCCESS).and("synchronizedVitamDate").lte(now));
    }
}
